#ifndef ANSTEUERUNG_H
#define ANSTEUERUNG_H
#include "CmdInterface.h"
#include "drive.h"
#include "Umwandlungsroutinen.h"
#include "AMS5812.h"

// Zuweisung der Arduino-Ausgänge zu den Treiber-Pins
#define X_STEP_PIN    54
#define X_DIR_PIN     55
#define X_ENABLE_PIN  38
#define Y_STEP_PIN    46
#define Y_DIR_PIN     48
#define Y_ENABLE_PIN  62

#define Poti 4
#define SCHALTERPIN 3
#define LEDPIN      13

#define HANDBETRIEB  LOW
#define AUTOMATIK    HIGH

#define POTIMIN  9600
#define POTIRUNNING 10000

// states
#define WAIT4RUN 0
#define RUNNING  1
#define STOPPING 2

////////////////// AMS5812 #Defines



//////////////////

// globals
CmdInterface cmd;
int state = WAIT4RUN;  
int x;
unsigned long a = 0;
unsigned long potiwert, clk;
bool temp = 0;
COMMAND command;



bool zustand = LOW;
bool schalterwert;
bool cmdblock = true;
bool potiblock = true;
int speed;

//bool temp = 0;

Drive Antrieb(X_STEP_PIN, X_DIR_PIN, X_ENABLE_PIN, Y_STEP_PIN, Y_DIR_PIN, Y_ENABLE_PIN);

void setup()
{
  cmd.Init(9600); // Einstellen der Übertragungsrate auf 9600 Baud
  //pinMode(Wahlschalter, INPUT);
  pinMode(SCHALTERPIN, INPUT_PULLUP);
  pinMode(LEDPIN, OUTPUT); 
  clk = millis();
  Wire.begin();  

  // reset
  TCCR5A = 0; // set TCCR1A register to 0
  TCCR5B = 0; // set TCCR1B register to 0
  TCNT5  = 0; // reset counter value
  
  OCR5A = Abtastfrequenz; // compare match register

  // set prescaler
  TCCR5B |= (1 << CS12) | (1 << CS10);   
  
  TCCR5B |= (1 << WGM12); // turn on CTC mode
  TIMSK5 |= (1 << OCIE1A); // enable timer compare interrupt
  
}                                                                            

// Interrupt service routine  Für den Schrittmotor
ISR(TIMER1_COMPA_vect){ 
  OCR1A = Antrieb.GetDrvTicks();   // Count Register neu setzen 
  
  Antrieb.OneStep();
}




ISR(TIMER5_COMPA_vect) { // für den sensor AMS5812 //  ISR_NOBLOCK vlt

  float Druck = GetAMSSensor();  
  //Serial.print(Druck); // send current value of i to the pc
  //Serial.println("  Druck in mBar vom TIMER");
  //Serial.print(millis());
  //Serial.println("  Vergange MilliSekunden");
}


void loop()
{

////////////// Funktionsaufruf für den AMS5812 Sensor code

float DruckwertAMS = AMSSensor();         //  später void
 
 //Serial.print(DruckwertAMS);
 //Serial.println("  Druck in mBar vom DRUCKwertAMS");
 //Serial.println("Druck in MilliBar");

 float Druck = GetAMSSensor();
 //Serial.print(Druck); // send current value of i to the pc
 //Serial.println("  Druck in mBar von GetAMSSensor()");
 
 

//////////////

  
  if(zustand != (schalterwert = digitalRead(SCHALTERPIN)))
   {
     zustand = schalterwert;
     
     potiblock = true;
     cmdblock = true;
     speed = 0;
     
     if(zustand == HANDBETRIEB) Serial.println("Handbetrieb!");
     if(zustand == AUTOMATIK)   Serial.println("Automatik!");
   }
      
   if((Antrieb.enable == !ENABLED) && (potiwert <= 10) && (zustand == HANDBETRIEB))
   { 
     potiblock = false; //Änderung: stopped zu !stopping // poti zu Potiwert
   }
   
   if((Antrieb.enable == !ENABLED) && (zustand == AUTOMATIK))
   {   
     cmdblock = false; //Änderung: stopped zu !stopping
   }
   
   // Wenn Poti nicht blockiert
   if(!potiblock && millis() - clk > 10)
   {
     speed = PotiToSpeed(readspeed());  // Potiwert einlesen
     clk = millis();       // Vergleichszeit neu setzen
   }
   
  /*if(millis() - old >= 1000)
  {
    old = millis();
   //Serial.print (NbTopsFan, DEC);
    //Serial.print (" Frequenz\n");
    Calc = ((NbTopsFan * 60) / 4.8); //(Pulse frequency x 60) / 7.5Q, = flow rate in L/hour / 
    NbTopsFan = 0;   //Set NbTops to 0 ready for calculations   // setzt die Die Variable zum aufzählen der Rechtecksignale auf null
     
    //Serial.print (Calc, DEC); //Prints the number calculated above / gibt den Wert in "Calc" als Dezimalzahl aus
    //Serial.print (" L/min\r\n"); //Prints "L/hour" and returns a  new line / Gibt was da steht aus
  }*/

     
    if(!cmdblock)
    {
       if(cmd.IsCommandAvailable()) 
         command = cmd.GetCommand();
       
       speed = CmdToSpeed(command);       
    }

       Antrieb.SetSpeed(speed);


  if(state == WAIT4RUN){ // Motor starten? 
    if((zustand == AUTOMATIK && command.keyword == 'N') || (zustand == HANDBETRIEB && speed > POTIMIN + 40))
    {
      Serial.println("RUNNING");
      state = RUNNING; 
      Antrieb.Start();                       // asynchron (Motor fährt danach hoch)
    }
  }
  
  if(state == RUNNING){  // Motor anhalten?
    if((zustand == AUTOMATIK && command.keyword == 'S') || (zustand == HANDBETRIEB && speed <= POTIMIN + 40))
    {
      Serial.println("STOPPING");
      state = STOPPING; 
      Antrieb.Stop();
    }    // asynchron 
  }

  if(state == STOPPING && !Antrieb.enable){
    Serial.println("WAIT4RUN");
    state = WAIT4RUN; 
  }

   
  
}  

unsigned int readspeed()
{
  unsigned int speed = (analogRead(Poti));
  if((int)speed <= 0) x = 0;
  else x = 1;
  return speed;
}



#endif
