// stepper.cpp


#include "CmdInterface.h"
 
// Standardkonstruktor, Buffercounter wird inizialisiert

CmdInterface::CmdInterface() : bufferCtr(0)
{   
}

void CmdInterface::Init(unsigned int baudrate)
{
   // Baudrate auswählen und für die Schnittestelle setzen
   switch(baudrate)
   {
     case 300:
     case 600:
     case 1200:
     case 2400:
     case 4800:
     case 9600:
     case 14400:
     case 19200:
     case 28800:
     case 38400:
     case 57600:
     case 115200:
       Serial.begin(baudrate);
       break;
     default:
       Serial.begin(9600);
       break;
   }
}

bool CmdInterface::IsCommandAvailable()
{
  char temp;
  
  while(Serial.available()) // Abfrage, ob Zeichen in seriellem Buffer verfügbar
  {
      temp = Serial.read(); // Einlesen des Zeichens aus dem seriellen Buffer
    
      if(temp == 13)  // Wert bei Abschluss einer Befehlszeile   (Carriage Return)
      {
          cmdBuffer[bufferCtr] = 0; // Letztes Bufferelement ist Null
          DecodeCommand();
          for(int i = 0; i < bufferCtr; i++)  cmdBuffer[i] = 0; // Buffer wird geleert
          bufferCtr = 0; // Zeichenbuffer - Zähler wird zurückgesetzt
          return true; // Anfrage nach vorhandenem Befehl bestätigen  
      }
      else
      {
        // Abbruch wenn die Zeilenlänge überschritten wird
        if( bufferCtr < STRLEN)
        {
            cmdBuffer[bufferCtr] = temp;  // Zeichenbuffer füllen
            bufferCtr++;
        } 
      }
  }
  
  return false;
}

void CmdInterface::DecodeCommand()
{
    unsigned int wPos = 0, ePos = 0; // Positionen der Zeichen zurücksetzen
    
    for(int i = 0; ; i++)
    {
        // Einzelne Worte separieren
        if( cmdBuffer[i] != ' ' && cmdBuffer[i] != 0)
        {
            if(ePos < WORDLEN)
            {
               cmdWord[wPos][ePos] = cmdBuffer[i]; // Alle Zeichen eines Wortes einlesen
               ePos++;
            }
        }
        else
        {
            cmdWord[wPos][ePos] = 0; // Wort mit Null als letztes Element abschließen 
            ePos = 0;
            wPos++;
            
            // Abbruch nach Überschreitung der Worte oder bei Ende des Zeichenbuffers
            if( wPos > WORDCNT || cmdBuffer[i] == 0) break; 
        }
    }
    
    // Befehls-Packet wird befüllt
    input.keyword = cmdWord[0][0];           // Erstes Zeichen vom ersten Wort wird als Befehl gesetzt 
    input.parameter = ToInteger(cmdWord[1]); // Zweites Wort wird als zugehöriger Zahlenwert gesetzt
}

COMMAND CmdInterface::GetCommand()
{
   return input; // Befehls-Packet wird zurückgegeben
}

// Für erweiterte Befehlserkennung
bool CmdInterface::CompareString(char* a, char* b) const
{
   for( int i = 0; a[i]; i++)
   {
       if(a[i] != b[i]) return false;
   }
   return true; // Bestätigung wenn die Eingabe mit einem Befehl übereinstimmt
}

int CmdInterface::ToInteger(char* a)
{
    return atoi(a); // Umwandlung eines Strings in einen Integerwert
}
