// drive.h

#include "drive.h"

Drive::Drive(int x_Step_Pin, int x_Dir_Pin, int x_Enable_Pin,int y_Step_Pin, int y_Dir_Pin, int y_Enable_Pin )
{
  motor1 = new StepperV1(x_Step_Pin, x_Dir_Pin, x_Enable_Pin);
  motor2 = new StepperV1(y_Step_Pin, y_Dir_Pin, y_Enable_Pin);
  
  motor1->SetDirection(FORWARD);
  motor2->SetDirection(!FORWARD);
  Disable();
  Stop();
}

Drive::~Drive()
{
  delete motor1;
  delete motor2;
}

void Drive::Enable()
{
  enable = ENABLED;
  
  motor1->SetEnable(ENABLED);
  motor2->SetEnable(ENABLED);
}

void Drive::Disable()
{
  motor1->SetEnable(!ENABLED);
  motor2->SetEnable(!ENABLED);
  
  TCCR1B &= ~(1 << CS10);    // Timer abschalten; 
  stopping = false;
  enable = !ENABLED; 
}

void Drive::Stop()
{
  stopping = true; 
  sps_soll = SPS_MIN; 
}

void Drive::Start()
 {
  Enable();
  
  sps = SPS_MIN;             // Startgeschwindigkeit
  ticks = CPU/sps;
  
  //Timer 1 initialisieren
  noInterrupts();            // alle Interrupts unterbinden
  TCCR1A = 0;                // Timer Couter Register Counter Register A
  TCCR1B = 0;                // Timer Counter Control Register B
  TCNT1 = 0;                 // Zähler auf 0 setzen
  TCCR1B |= (1 << WGM12);    // CTC mode: Clear Timer on Compare match.
  TCCR1B |= (1 << CS10);     // prescaler: 0=Timer aus; 1=CS10; 8=CS11; 64=CS10|CS11; 256=CS12; 1024=CS12|CS10
  TIMSK1 |= _BV(OCIE1A);     // ISR-Vektor TIMER1_COMPA_vect im TIMER Interrupt Mask Register auswählen
  OCR1A = ticks;             // Count Register setzen
  interrupts();              // wieder erlauben
 }
 
 void Drive::SetSpeed(unsigned int v_new){
   
   if(!stopping)
      sps_soll = v_new;        // Sollwert auf Poti- bzw. Cmd-Wert setzen
   else
    if(sps<SPS_MIN+2*ACCEL){   // Anhaltevorgang überwachen  
      Disable(); 
      return; 
    }
  // Wenn Ist-Wert sich dem Soll-Wert genug angenähert hat, keine weitere Geschwindigkeitsanpassung
  if(abs(sps-sps_soll)<ACCEL+1) // Geschwindigkeit halten? //abs damit neg. wert zu pos. wert wird?
    return; 
  if(0==divisor++%6)
    sps += (sps>sps_soll)?-ACCEL:ACCEL; // schneller oder langsamer  // Anpassen von Ist-Wert an Soll-Wert alle 6 Durchläufe
  else return; 
  v_new =  CPU/sps;          // dauert 256 Zyklen! http://www.atmel.com/Images/doc0936.pdf
  noInterrupts();            // critical section, weil 16/Bit-Zuweisung
    ticks = v_new;   
  interrupts();   
}    

 void Drive::OneStep()
 {
   motor1->SetStepPin(HIGH);
   motor2->SetStepPin(HIGH);
   motor1->SetStepPin(LOW);
   motor2->SetStepPin(LOW);
 }
 
 unsigned int Drive::GetDrvTicks()
 {
   return ticks;
 }
